@@ -15,8 +15,9 @@
 use std::collections::{HashMap, HashSet, VecDeque};
 use std::convert::TryFrom;
 use std::fmt;
-use std::fs;
-use std::path::{Path, PathBuf};
+use std::path::Path;
+#[cfg(feature = "diesel-receipt-store")]
+use std::str::FromStr;
 use std::sync::{
     mpsc::{channel, Receiver, RecvTimeoutError, Sender},
     Arc, RwLock,
@@ -24,7 +25,10 @@ use std::sync::{
 use std::time::{Duration, Instant, SystemTime};
 
 use protobuf::Message;
-use sawtooth::store::{lmdb::LmdbOrderedStore, receipt_store::TransactionReceiptStore};
+#[cfg(feature = "diesel-receipt-store")]
+use sawtooth::receipt::store::ReceiptStore;
+#[cfg(not(feature = "diesel-receipt-store"))]
+use sawtooth::store::receipt_store::TransactionReceiptStore;
 use sawtooth_sabre::{
     handler::SabreTransactionHandler, ADMINISTRATORS_SETTING_ADDRESS, ADMINISTRATORS_SETTING_KEY,
 };
@@ -62,6 +66,12 @@ const ITER_CACHE_SIZE: usize = 64;
 const COMPLETED_BATCH_INFO_ITER_RETRY_MILLIS: u64 = 100;
 const DEFAULT_BATCH_HISTORY_SIZE: usize = 100;
 
+#[cfg(not(feature = "diesel-receipt-store"))]
+type ScabbardReceiptStore = Arc<RwLock<TransactionReceiptStore>>;
+
+#[cfg(feature = "diesel-receipt-store")]
+type ScabbardReceiptStore = Arc<RwLock<dyn ReceiptStore>>;
+
 /// Iterator over entries in a Scabbard service's state
 pub type StateIter = Box<dyn Iterator<Item = Result<(String, Vec<u8>), ScabbardStateError>>>;
 
@@ -70,29 +80,23 @@ pub struct ScabbardState {
     context_manager: ContextManager,
     executor: Executor,
     current_state_root: String,
-    transaction_receipt_store: Arc<RwLock<TransactionReceiptStore>>,
+    receipt_store: ScabbardReceiptStore,
     pending_changes: Option<(String, Vec<TransactionReceipt>)>,
     event_subscribers: Vec<Box<dyn StateSubscriber>>,
     batch_history: BatchHistory,
-    state_db_file: PathBuf,
-    receipt_db_file: PathBuf,
 }
 
 impl ScabbardState {
     pub fn new(
         state_db_path: &Path,
         state_db_size: usize,
-        receipt_db_path: &Path,
-        receipt_db_size: usize,
+        receipt_store: ScabbardReceiptStore,
         admin_keys: Vec<String>,
     ) -> Result<Self, ScabbardStateError> {
         // Initialize the database
         let mut indexes = INDEXES.to_vec();
         indexes.push(CURRENT_STATE_ROOT_INDEX);
-        let state_db_file = state_db_path.to_path_buf();
-        let receipt_db_file = receipt_db_path.to_path_buf();
         let state_db_path = state_db_path.with_extension("lmdb");
-        let receipt_db_path = receipt_db_path.with_extension("lmdb");
         let db = Box::new(LmdbDatabase::new(
             LmdbContext::new(&state_db_path, indexes.len(), Some(state_db_size))?,
             &indexes,
@@ -152,17 +156,10 @@ impl ScabbardState {
             context_manager,
             executor,
             current_state_root,
-            transaction_receipt_store: Arc::new(RwLock::new(TransactionReceiptStore::new(
-                Box::new(
-                    LmdbOrderedStore::new(&receipt_db_path, Some(receipt_db_size))
-                        .map_err(|err| ScabbardStateError(err.to_string()))?,
-                ),
-            ))),
+            receipt_store,
             pending_changes: None,
             event_subscribers: vec![],
             batch_history: BatchHistory::new(),
-            state_db_file,
-            receipt_db_file,
         })
     }
 
@@ -318,7 +315,8 @@ impl ScabbardState {
                     .map(StateChangeEvent::try_from)
                     .collect::<Result<Vec<_>, _>>()?;
 
-                self.transaction_receipt_store
+                #[cfg(not(feature = "diesel-receipt-store"))]
+                self.receipt_store
                     .write()
                     .map_err(|err| {
                         ScabbardStateError(format!(
@@ -334,6 +332,23 @@ impl ScabbardState {
                         ))
                     })?;
 
+                #[cfg(feature = "diesel-receipt-store")]
+                self.receipt_store
+                    .write()
+                    .map_err(|err| {
+                        ScabbardStateError(format!(
+                            "transaction receipt store lock poisoned: {}",
+                            err
+                        ))
+                    })?
+                    .add_txn_receipts(txn_receipts)
+                    .map_err(|err| {
+                        ScabbardStateError(format!(
+                            "failed to add transaction receipts to store: {}",
+                            err
+                        ))
+                    })?;
+
                 for event in events {
                     self.event_subscribers.retain(|subscriber| {
                         match subscriber.handle_event(event.clone()) {
@@ -372,7 +387,11 @@ impl ScabbardState {
     }
 
     pub fn get_events_since(&self, event_id: Option<String>) -> Result<Events, ScabbardStateError> {
-        Events::new(self.transaction_receipt_store.clone(), event_id)
+        #[cfg(not(feature = "diesel-receipt-store"))]
+        let events = Events::new(self.receipt_store.clone(), event_id);
+        #[cfg(feature = "diesel-receipt-store")]
+        let events = Events::new(self.receipt_store.clone(), event_id);
+        events
     }
 
     pub fn add_subscriber(&mut self, subscriber: Box<dyn StateSubscriber>) {
@@ -382,28 +401,25 @@ impl ScabbardState {
     pub fn clear_subscribers(&mut self) {
         self.event_subscribers.clear();
     }
+}
 
-    /// Computes the files associated with the LMDB state, including lock files.
-    pub fn remove_db_files(&self) -> Result<(), ScabbardStateError> {
-        let state_db_path = self.state_db_file.with_extension("lmdb");
-        let state_db_lock_file_path = self.state_db_file.with_extension("lmdb-lock");
-        let receipt_db_path = self.receipt_db_file.with_extension("lmdb");
-        let receipt_db_lock_file_path = self.receipt_db_file.with_extension("lmdb-lock");
+/// The possible connection types and identifiers passed to the migrate command
+#[cfg(feature = "diesel-receipt-store")]
+pub enum ConnectionUri {
+    Postgres(String),
+    Sqlite(String),
+}
 
-        fs::remove_file(receipt_db_path.as_path()).map_err(|err| {
-            ScabbardStateError(format!("Unable to remove state LMDB file: {}", err))
-        })?;
-        fs::remove_file(receipt_db_lock_file_path.as_path()).map_err(|err| {
-            ScabbardStateError(format!("Unable to remove state LMDB file: {}", err))
-        })?;
-        fs::remove_file(state_db_path.as_path()).map_err(|err| {
-            ScabbardStateError(format!("Unable to remove receipt store LMDB file: {}", err))
-        })?;
-        fs::remove_file(state_db_lock_file_path.as_path()).map_err(|err| {
-            ScabbardStateError(format!("Unable to remove state LMDB file: {}", err))
-        })?;
+#[cfg(feature = "diesel-receipt-store")]
+impl FromStr for ConnectionUri {
+    type Err = ScabbardStateError;
 
-        Ok(())
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        if s.starts_with("postgres://") {
+            Ok(ConnectionUri::Postgres(s.into()))
+        } else {
+            Ok(ConnectionUri::Sqlite(s.into()))
+        }
     }
 }
 
@@ -503,6 +519,7 @@ enum EventQuery {
     Exhausted,
 }
 
+#[cfg(not(feature = "diesel-receipt-store"))]
 /// An iterator that wraps the `TransactionReceiptStore` and returns `StateChangeEvent`s using an
 /// in-memory cache.
 pub struct Events {
@@ -511,6 +528,7 @@ pub struct Events {
     cache: VecDeque<StateChangeEvent>,
 }
 
+#[cfg(not(feature = "diesel-receipt-store"))]
 impl Events {
     fn new(
         transaction_receipt_store: Arc<RwLock<TransactionReceiptStore>>,
@@ -564,6 +582,73 @@ impl Events {
     }
 }
 
+#[cfg(feature = "diesel-receipt-store")]
+/// An iterator that wraps the `ReceiptStore` and returns `StateChangeEvent`s using an
+/// in-memory cache.
+pub struct Events {
+    receipt_store: Arc<RwLock<dyn ReceiptStore>>,
+    query: EventQuery,
+    cache: VecDeque<StateChangeEvent>,
+}
+
+#[cfg(feature = "diesel-receipt-store")]
+impl Events {
+    fn new(
+        receipt_store: Arc<RwLock<dyn ReceiptStore>>,
+        start_id: Option<String>,
+    ) -> Result<Self, ScabbardStateError> {
+        let mut iter = Events {
+            receipt_store,
+            query: EventQuery::Fetch(start_id),
+            cache: VecDeque::default(),
+        };
+        iter.reload_cache()?;
+        Ok(iter)
+    }
+
+    fn reload_cache(&mut self) -> Result<(), ScabbardStateError> {
+        match self.query {
+            EventQuery::Fetch(ref start_id) => {
+                let receipt_store = self.receipt_store.read().map_err(|err| {
+                    ScabbardStateError(format!("transaction receipt store lock poisoned: {}", err))
+                })?;
+
+                self.cache = if let Some(id) = start_id.as_ref() {
+                    receipt_store.list_receipts_since(Some(id.clone()))
+                } else {
+                    receipt_store.list_receipts_since(None)
+                }
+                .map_err(|err| {
+                    ScabbardStateError(format!(
+                        "failed to get transaction receipts from store: {}",
+                        err
+                    ))
+                })?
+                .take(ITER_CACHE_SIZE)
+                .map(|res| match res {
+                    Ok(receipt) => StateChangeEvent::try_from(receipt),
+                    Err(err) => {
+                        return Err(ScabbardStateError(format!(
+                            "failed to get transaction receipt: {}",
+                            err
+                        )))
+                    }
+                })
+                .collect::<Result<VecDeque<_>, _>>()?;
+
+                self.query = self
+                    .cache
+                    .back()
+                    .map(|event| EventQuery::Fetch(Some(event.id.clone())))
+                    .unwrap_or(EventQuery::Exhausted);
+
+                Ok(())
+            }
+            EventQuery::Exhausted => Ok(()),
+        }
+    }
+}
+
 impl Iterator for Events {
     type Item = StateChangeEvent;
 
@@ -891,6 +976,17 @@ mod tests {
     use std::path::PathBuf;
 
     use cylinder::{secp256k1::Secp256k1Context, Context};
+    #[cfg(feature = "diesel-receipt-store")]
+    use diesel::{
+        r2d2::{ConnectionManager, Pool},
+        sqlite::SqliteConnection,
+    };
+    #[cfg(feature = "diesel-receipt-store")]
+    use sawtooth::migrations::run_sqlite_migrations;
+    #[cfg(feature = "diesel-receipt-store")]
+    use sawtooth::receipt::store::diesel::DieselReceiptStore;
+    #[cfg(not(feature = "diesel-receipt-store"))]
+    use sawtooth::store::lmdb::LmdbOrderedStore;
     use tempdir::TempDir;
     use transact::{
         families::command::make_command_transaction,
@@ -905,29 +1001,49 @@ mod tests {
     /// Verify that an empty receipt store returns an empty iterator
     #[test]
     fn empty_event_iterator() {
-        let paths = StatePaths::new("empty_event_iterator");
+        #[cfg(not(feature = "diesel-receipt-store"))]
+        {
+            let paths = StatePaths::new("empty_event_iterator");
 
-        let transaction_receipt_store =
-            Arc::new(RwLock::new(TransactionReceiptStore::new(Box::new(
-                LmdbOrderedStore::new(&paths.receipt_db_path, Some(TEMP_DB_SIZE))
-                    .expect("Failed to create LMDB store"),
-            ))));
+            let transaction_receipt_store =
+                Arc::new(RwLock::new(TransactionReceiptStore::new(Box::new(
+                    LmdbOrderedStore::new(&paths.receipt_db_path, Some(TEMP_DB_SIZE))
+                        .expect("Failed to create LMDB store"),
+                ))));
 
-        // Test without a specified start
-        let all_events = Events::new(transaction_receipt_store.clone(), None)
-            .expect("failed to get iterator for all events");
-        let all_event_ids = all_events.map(|event| event.id.clone()).collect::<Vec<_>>();
-        assert!(
-            all_event_ids.is_empty(),
-            "All events should have been empty"
-        );
+            #[cfg(not(feature = "diesel-receipt-store"))]
+            // Test without a specified start
+            let all_events = Events::new(transaction_receipt_store.clone(), None)
+                .expect("failed to get iterator for all events");
+            let all_event_ids = all_events.map(|event| event.id.clone()).collect::<Vec<_>>();
+
+            assert!(
+                all_event_ids.is_empty(),
+                "All events should have been empty"
+            );
+        }
+
+        #[cfg(feature = "diesel-receipt-store")]
+        {
+            let pool = create_connection_pool_and_migrate(":memory:".to_string());
+
+            let receipt_store = Arc::new(RwLock::new(DieselReceiptStore::new(pool)));
+
+            // Test without a specified start
+            let all_events =
+                Events::new(receipt_store, None).expect("failed to get iterator for all events");
+            let all_event_ids = all_events.map(|event| event.id.clone()).collect::<Vec<_>>();
+
+            assert!(
+                all_event_ids.is_empty(),
+                "All events should have been empty"
+            );
+        }
     }
 
     /// Verify that the event iterator works as expected.
     #[test]
     fn event_iterator() {
-        let paths = StatePaths::new("event_iterator");
-
         let receipts = vec![
             mock_transaction_receipt("ab"),
             mock_transaction_receipt("cd"),
@@ -938,34 +1054,67 @@ mod tests {
             .map(|receipt| receipt.transaction_id.clone())
             .collect::<Vec<_>>();
 
-        let transaction_receipt_store =
-            Arc::new(RwLock::new(TransactionReceiptStore::new(Box::new(
-                LmdbOrderedStore::new(&paths.receipt_db_path, Some(TEMP_DB_SIZE))
-                    .expect("Failed to create LMDB store"),
-            ))));
+        #[cfg(not(feature = "diesel-receipt-store"))]
+        {
+            let paths = StatePaths::new("event_iterator");
 
-        transaction_receipt_store
-            .write()
-            .expect("failed to get write lock")
-            .append(receipts.clone())
-            .expect("failed to add receipts to store");
+            let transaction_receipt_store =
+                Arc::new(RwLock::new(TransactionReceiptStore::new(Box::new(
+                    LmdbOrderedStore::new(&paths.receipt_db_path, Some(TEMP_DB_SIZE))
+                        .expect("Failed to create LMDB store"),
+                ))));
 
-        // Test without a specified start
-        let all_events = Events::new(transaction_receipt_store.clone(), None)
-            .expect("failed to get iterator for all events");
-        let all_event_ids = all_events.map(|event| event.id.clone()).collect::<Vec<_>>();
-        assert_eq!(all_event_ids, receipt_ids);
+            transaction_receipt_store
+                .write()
+                .expect("failed to get write lock")
+                .append(receipts.clone())
+                .expect("failed to add receipts to store");
 
-        // Test with a specified start
-        let some_events = Events::new(
-            transaction_receipt_store.clone(),
-            Some(receipt_ids[0].clone()),
-        )
-        .expect("failed to get iterator for some events");
-        let some_event_ids = some_events
-            .map(|event| event.id.clone())
-            .collect::<Vec<_>>();
-        assert_eq!(some_event_ids, receipt_ids[1..].to_vec());
+            // Test without a specified start
+            let all_events = Events::new(transaction_receipt_store.clone(), None)
+                .expect("failed to get iterator for all events");
+
+            let all_event_ids = all_events.map(|event| event.id.clone()).collect::<Vec<_>>();
+            assert_eq!(all_event_ids, receipt_ids);
+
+            // Test with a specified start
+            let some_events = Events::new(transaction_receipt_store, Some(receipt_ids[0].clone()))
+                .expect("failed to get iterator for some events");
+
+            let some_event_ids = some_events
+                .map(|event| event.id.clone())
+                .collect::<Vec<_>>();
+            assert_eq!(some_event_ids, receipt_ids[1..].to_vec());
+        }
+
+        #[cfg(feature = "diesel-receipt-store")]
+        {
+            let pool = create_connection_pool_and_migrate(":memory:".to_string());
+
+            let receipt_store = Arc::new(RwLock::new(DieselReceiptStore::new(pool)));
+
+            receipt_store
+                .write()
+                .expect("failed to get write lock")
+                .add_txn_receipts(receipts.clone())
+                .expect("failed to add receipts to store");
+
+            // Test without a specified start
+            let all_events = Events::new(receipt_store.clone(), None)
+                .expect("failed to get iterator for all events");
+
+            let all_event_ids = all_events.map(|event| event.id.clone()).collect::<Vec<_>>();
+            assert_eq!(all_event_ids, receipt_ids);
+
+            // Test with a specified start
+            let some_events = Events::new(receipt_store, Some(receipt_ids[0].clone()))
+                .expect("failed to get iterator for some events");
+
+            let some_event_ids = some_events
+                .map(|event| event.id.clone())
+                .collect::<Vec<_>>();
+            assert_eq!(some_event_ids, receipt_ids[1..].to_vec());
+        }
     }
 
     /// Verify that the `ScabbardState::get_state_at_address` method works properly.
@@ -979,14 +1128,23 @@ mod tests {
     fn get_state_at_address() {
         // Initialize state
         let paths = StatePaths::new("get_state_at_address");
-        let mut state = ScabbardState::new(
-            &paths.state_db_path,
-            TEMP_DB_SIZE,
-            &paths.receipt_db_path,
-            TEMP_DB_SIZE,
-            vec![],
-        )
-        .expect("Failed to initialize state");
+
+        #[cfg(not(feature = "diesel-receipt-store"))]
+        let receipt_store = Arc::new(RwLock::new(TransactionReceiptStore::new(Box::new(
+            LmdbOrderedStore::new(
+                &StatePaths::new("state_at_address").receipt_db_path,
+                Some(TEMP_DB_SIZE),
+            )
+            .expect("Failed to create LMDB store"),
+        ))));
+        #[cfg(feature = "diesel-receipt-store")]
+        let receipt_store = Arc::new(RwLock::new(DieselReceiptStore::new(
+            create_connection_pool_and_migrate(":memory:".to_string()),
+        )));
+
+        let mut state =
+            ScabbardState::new(&paths.state_db_path, TEMP_DB_SIZE, receipt_store, vec![])
+                .expect("Failed to initialize state");
 
         // Set a value in state
         let address = "abcdef".to_string();
@@ -1043,16 +1201,24 @@ mod tests {
     ///    that no entries are returned (the iterator is empty).
     #[test]
     fn get_state_with_prefix() {
-        // Initialize state
         let paths = StatePaths::new("get_state_at_address");
-        let mut state = ScabbardState::new(
-            &paths.state_db_path,
-            TEMP_DB_SIZE,
-            &paths.receipt_db_path,
-            TEMP_DB_SIZE,
-            vec![],
-        )
-        .expect("Failed to initialize state");
+
+        #[cfg(not(feature = "diesel-receipt-store"))]
+        let receipt_store = Arc::new(RwLock::new(TransactionReceiptStore::new(Box::new(
+            LmdbOrderedStore::new(
+                &StatePaths::new("state_with_prefix").receipt_db_path,
+                Some(TEMP_DB_SIZE),
+            )
+            .expect("Failed to create LMDB store"),
+        ))));
+        #[cfg(feature = "diesel-receipt-store")]
+        let receipt_store = Arc::new(RwLock::new(DieselReceiptStore::new(
+            create_connection_pool_and_migrate(":memory:".to_string()),
+        )));
+
+        let mut state =
+            ScabbardState::new(&paths.state_db_path, TEMP_DB_SIZE, receipt_store, vec![])
+                .expect("Failed to initialize state");
 
         // Set some values in state
         let prefix = "abcdef".to_string();
@@ -1119,6 +1285,7 @@ mod tests {
     struct StatePaths {
         _temp_dir_handle: TempDir,
         pub state_db_path: PathBuf,
+        #[cfg(not(feature = "diesel-receipt-store"))]
         pub receipt_db_path: PathBuf,
     }
 
@@ -1126,10 +1293,12 @@ mod tests {
         fn new(prefix: &str) -> Self {
             let temp_dir = TempDir::new(prefix).expect("Failed to create temp dir");
             let state_db_path = temp_dir.path().join("state.lmdb");
+            #[cfg(not(feature = "diesel-receipt-store"))]
             let receipt_db_path = temp_dir.path().join("receipts.lmdb");
             Self {
                 _temp_dir_handle: temp_dir,
                 state_db_path,
+                #[cfg(not(feature = "diesel-receipt-store"))]
                 receipt_db_path,
             }
         }
@@ -1145,4 +1314,20 @@ mod tests {
             },
         }
     }
+
+    #[cfg(feature = "diesel-receipt-store")]
+    fn create_connection_pool_and_migrate(
+        connection_string: String,
+    ) -> Pool<ConnectionManager<SqliteConnection>> {
+        let connection_manager = ConnectionManager::<SqliteConnection>::new(connection_string);
+        let pool = Pool::builder()
+            .max_size(1)
+            .build(connection_manager)
+            .expect("Failed to build connection pool");
+
+        run_sqlite_migrations(&*pool.get().expect("Failed to get connection for migrations"))
+            .expect("Failed to run migrations");
+
+        pool
+    }
 }
